// ignore_for_file: use_key_in_widget_constructors

import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:aretmis_cupps_runner/data.dart';
import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:artemis_cupps/classes/Requests/device_query_request.dart';
import 'package:dart_ipify/dart_ipify.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:get/get.dart';
import 'package:network_info_plus/network_info_plus.dart';
import 'package:scoped_model/scoped_model.dart';
import 'global/MainModel.dart';
import 'screens/CuppsController.dart';
import 'screens/bc_device_screen.dart';
import 'screens/bp_device_screen.dart';
import 'screens/bt_device_screen.dart';
import 'screens/log_screen.dart';
import 'screens/pr_device_screen.dart';
import 'screens/oc_device_screen.dart';
import 'screens/ms_device_screen.dart';
import 'screens/zi_device_screen.dart';
import 'screens/zl_device_screen.dart';
import 'screens/be_device_screen.dart';
import 'screens/bg_device_screen.dart';
import 'screens/dd_device_screen.dart';

void main() {
  WidgetsFlutterBinding.ensureInitialized();
  runZonedGuarded(() {
    runApp(MyApp());
  }, (error, stackTrace) {
    log('runZonedGuarded:  $error');
    // FirebaseCrashlytics.instance.recordError(error, stackTrace);
  });
}

class MyApp extends StatelessWidget {
  MyApp({Key? key}) : super(key: key);
  final MainModel _model = MainModel();
  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {

    return ScopedModel<MainModel>(
      model: _model,
      child: GetMaterialApp(
        title: 'Flutter Demo',
        debugShowCheckedModeBanner: false,
        theme: ThemeData(
          // This is the theme of your application.
          //
          // Try running your application with "flutter run". You'll see the
          // application has a blue toolbar. Then, without quitting the app, try
          // changing the primarySwatch below to Colors.green and then invoke
          // "hot reload" (press "r" in the console where you ran "flutter run",
          // or simply save your changes to "hot reload" in a Flutter IDE).
          // Notice that the counter didn't reset back to zero; the application
          // is not restarted.
          primarySwatch: Colors.blue,
        ),
        home: CuppsView(CuppsController(_model)),
      ),
    );
  }
}



class CuppsView extends StatefulWidget {
  final CuppsController myCuppsController;

  CuppsView(this.myCuppsController);

  @override
  State<StatefulWidget> createState() {
    return _CuppsViewState();
  }
}

class _CuppsViewState extends State<CuppsView> {
  TextEditingController ipC = TextEditingController(text: "");
  TextEditingController portC = TextEditingController(text: "");
  bool loading = false;
  bool connecting = false;
  bool authing = false;
  bool initializingDevices = false;
  // String status = "";

  @override
  void initState() {
    WidgetsBinding.instance!.addPostFrameCallback((timeStamp) {
      CUPPS.setDeviceTypes([
        DeviceType.bp,
        DeviceType.bt,
      ]);
    });
    super.initState();
  }

  goToDeviceDetails(dynamic device) {
    switch (device.runtimeType) {
      case BpDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BpDeviceScreen(device: device)));
        break;
      case BtDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BtDeviceScreen(device: device)));
        break;
      case BcDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BcDeviceScreen(device: device)));
        break;
      case BeDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BeDeviceScreen(device: device)));
        break;
      case OcDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => OcDeviceScreen(device: device)));
        break;
      case MsDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => MsDeviceScreen(device: device)));
        break;
      case PrDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => PrDeviceScreen(device: device)));
        break;
      case BgDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => BgDeviceScreen(device: device)));
        break;
      case DdDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => DdDeviceScreen(device: device)));
        break;
      case ZlDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => ZlDeviceScreen(device: device)));
        break;
      case ZiDevice:
        Navigator.push(context, MaterialPageRoute(builder: (c) => ZiDeviceScreen(device: device)));
        break;
    }
  }

  @override
  Widget build(BuildContext context) {
    return ScopedModelDescendant<MainModel>(builder: (BuildContext context, Widget? child, MainModel model) {
      return Scaffold(
        appBar: AppBar(
          title: const Text("CUPPS"),
          actions: [
            IconButton(
                onPressed: () async {
                  // Navigator.push(context, MaterialPageRoute(builder: (c) => const LogScreen()));
                  AuthenticateResponse res = AuthenticateResponse.fromXML(XMLExamples.authResLocal);
                  print(res.toXML());
                },
                icon: const Icon(Icons.bug_report_outlined))
          ],
        ),
        backgroundColor: Colors.white,
        body: Column(
          children: [
            Container(
              width: Get.width,
              color: Colors.blue,
              child: Center(
                child: Text(
                  "${model.platformStatus}",
                  style: TextStyle(color: Colors.white),
                ),
              ),
            ),
            const Divider(),
            Container(
              padding: const EdgeInsets.symmetric(horizontal: 24),
              width: MediaQuery.of(context).size.width,
              color: Colors.white,
              child: Column(
                children: [
                  model.platformConnection
                      ? Row(
                    children: [
                      Expanded(child: Text("Connected to ${ipC.text}:${portC.text}")),
                      TextButton(
                          onPressed: () async {
                            widget.myCuppsController.disconnectCupps();
                          },
                          child: const Text("Disconnect")),
                    ],
                  ): Row(
                    children: [
                      Expanded(child: Text("Cupps Platform Address: ${ipC.text} : ${portC.text}")),
                      TextButton(
                          onPressed: () async {
                            widget.myCuppsController.initializeCupps();
                          },
                          child: const Text("Connect")),
                    ],
                  ),

                  //     : Row(
                  //         children: [
                  //           Expanded(
                  //               flex: 4,
                  //               child: CupertinoTextField(
                  //                 keyboardType: const TextInputType.numberWithOptions(signed: true, decimal: true),
                  //                 controller: ipC,
                  //                 placeholder: "ip e.g. 192.168.1.7",
                  //               )),
                  //           const SizedBox(width: 12),
                  //           Expanded(
                  //               flex: 2,
                  //               child: CupertinoTextField(
                  //
                  //                 keyboardType: const TextInputType.numberWithOptions(signed: true),
                  //                 controller: portC,
                  //                 placeholder: "port e.g. 7535",
                  //               )),
                  //           const SizedBox(width: 12),
                  //           TextButton(
                  //               onPressed: () async {
                  //                 setState(() {
                  //                   connecting = true;
                  //                   status = "Connecting to platform...";
                  //                 });
                  //                 CUPPS
                  //                     .connectToPlatform(
                  //                         ip: ipC.text,
                  //                         port: int.tryParse(portC.text) ?? 0,
                  //                         airlineCode: "TK",
                  //                         notifier: () {
                  //                           model.setPlatformDevices(CUPPS().platformDevices);
                  //                         },
                  //                         onPlatformConnectionChange: (bool v) {
                  //                           model.setPlatformConnection(v);
                  //                           model.setPlatformDevices(PlatformDevices.empty());
                  //                         })
                  //                     .then((value) async {
                  //                   setState(() {
                  //                     status = "";
                  //                     connecting = false;
                  //                   });
                  //                   if (value) {
                  //                     setState(() {
                  //                       authing = true;
                  //                       status = "Authenticating...";
                  //                     });
                  //                     await widget.myCuppsController.saveAddressToSP(ipC.text, portC.text);
                  //                     CUPPS.authenticate().then((value) {
                  //                       setState(() {
                  //                         status = "";
                  //                         authing = false;
                  //                       });
                  //                       if (value) {
                  //                         setState(() {
                  //                           status = "Initializing devices...";
                  //                           initializingDevices = true;
                  //                         });
                  //                         CUPPS.connectToAvailableNeededDevices().then((value) {
                  //                           setState(() {
                  //                             status = "";
                  //                             initializingDevices = false;
                  //                           });
                  //                           widget.myCuppsController.lockAllAvailable();
                  //                         });
                  //                       }
                  //                     });
                  //                     // var req = InterfaceLevelsAvailableRequest(
                  //                     //     hsXsdVersion: '01.03', messageID: CUPPS().currentMessageID);
                  //                     // CUPPS.send(xml: req.toXML());
                  //                   }
                  //                 });
                  //               },
                  //               child: connecting ? const Text("...") : const Text("Connect")),
                  //         ],
                  //       ),
                ],
              ),
            ),
            const Divider(),
            const SizedBox(width: 12),
            // TextButton(
            //     onPressed: () async {
            //       SocketService.initialize(ip: "192.168.1.7", port: 7535);
            //     },
            //     child: connecting ? const Text("...") : const Text("Connect")),
            // TextButton(
            //     onPressed: () async {
            //       InterfaceLevelsAvailableRequest req = InterfaceLevelsAvailableRequest(messageID: 1, hsXsdVersion: "01.03");
            //
            //       final result = await SocketService.sendSocketMessage(req.toXML(), 1).timeout(Duration(seconds: 4), onTimeout: () {
            //         print("timeout");
            //         return "timeOut";
            //       });
            //       print(result);
            //     },
            //     child: const Text("Send")),
            const Divider(),
            Expanded(
                child: GridView(
                  padding: const EdgeInsets.symmetric(horizontal: 12),
                  gridDelegate:
                  const SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 3, crossAxisSpacing: 12, mainAxisSpacing: 12),
                  children: model.platformDevices.bpDevices
                      .map((e) => DeviceWidget(
                    device: e,
                    onSelect: () async {
                      goToDeviceDetails(e);
                    },
                  ))
                      .toList() +
                      model.platformDevices.btDevices
                          .map((e) => DeviceWidget(
                        device: e,
                        onSelect: () {
                          goToDeviceDetails(e);
                        },
                      ))
                          .toList() +
                      model.platformDevices.bcDevices
                          .map((e) => DeviceWidget(
                        device: e,
                        onSelect: () {
                          goToDeviceDetails(e);
                        },
                      ))
                          .toList() +
                      model.platformDevices.bgDevices
                          .map((e) => DeviceWidget(
                        device: e,
                        onSelect: () {
                          goToDeviceDetails(e);
                        },
                      ))
                          .toList() +
                      model.platformDevices.beDevices
                          .map((e) => DeviceWidget(
                        device: e,
                        onSelect: () {
                          goToDeviceDetails(e);
                        },
                      ))
                          .toList() +
                      // model.platformDevices.ddDevices
                      //     .map((e) => DeviceWidget(
                      //           device: e,
                      //           onSelect: () {
                      //             goToDeviceDetails(e);
                      //           },
                      //         ))
                      //     .toList() +
                      model.platformDevices.msDevices
                          .map((e) => DeviceWidget(
                        device: e,
                        onSelect: () {
                          goToDeviceDetails(e);
                        },
                      ))
                          .toList() +
                      model.platformDevices.prDevices
                          .map((e) => DeviceWidget(
                        device: e,
                        onSelect: () {
                          goToDeviceDetails(e);
                        },
                      ))
                          .toList() +
                      // model.platformDevices.ziDevices
                      //     .map((e) => DeviceWidget(
                      //           device: e,
                      //           onSelect: () {
                      //             goToDeviceDetails(e);
                      //           },
                      //         ))
                      //     .toList() +
                      // model.platformDevices.zlDevices
                      //     .map((e) => DeviceWidget(
                      //           device: e,
                      //           onSelect: () {
                      //             goToDeviceDetails(e);
                      //           },
                      //         ))
                      //     .toList() +
                      model.platformDevices.ocDevices
                          .map((e) => DeviceWidget(
                        device: e,
                        onSelect: () {
                          goToDeviceDetails(e);
                        },
                      ))
                          .toList(),
                )),
            (connecting || authing || initializingDevices)
                ? const SpinKitCubeGrid(
              color: Colors.blue,
              size: 40,
            )
                : Container(),
            ElevatedButton(
              onPressed: () async {
                final req = InterfaceLevelRequest(messageID: 100, level: '01.03');
                CUPPS.send(xml: req.toXML());

              },
              child: const Text("Session Error"),
            ),
            ElevatedButton(
              onPressed: () async {
                final req = DeviceQueryRequest(messageID: 100);
                CUPPS.send(xml: req.toXML());
              },
              child: const Text("Message ID Error"),
            ),
            FutureBuilder(
                future: SocketService.getLocalIp(),
                builder: (c, snapshot) {
                  if (snapshot.connectionState == ConnectionState.done || snapshot.hasData) {
                    var data = snapshot.data;
                    return Text("Device local IP $data");
                  }
                  return const Text("Loading ip");
                }),
            FutureBuilder(
                future: SocketService.getPublicIp(),
                builder: (c, snapshot) {
                  if (snapshot.connectionState == ConnectionState.done || snapshot.hasData) {
                    var data = snapshot.data;
                    return Text("Device public IP $data");
                  }
                  return const Text("Loading ip");
                }),
          ],
        ),
      );
    });
  }
}

class DeviceWidget extends StatelessWidget {
  final Device device;
  final Function onSelect;

  const DeviceWidget({required this.device, required this.onSelect});

  @override
  Widget build(BuildContext context) {
    ThemeData theme = Theme.of(context);
    return TextButton(
        onPressed: () {
          onSelect();
        },
        child: Image.asset(device.icon, package: 'artemis_cupps'));
  }
}

class DeviceStatusMessageWidget extends StatelessWidget {
  final Device device;

  const DeviceStatusMessageWidget({required this.device});

  @override
  Widget build(BuildContext context) {
    ThemeData theme = Theme.of(context);
    double width = Get.width;
    double height = Get.height;
    return Container(
        padding: EdgeInsets.symmetric(vertical: 8),
        width: Get.width,
        color: Colors.orange,
        child: Center(
            child: Text(
              "${device.statusMsg}",
              style: TextStyle(fontSize: 22,color: Colors.white),
            )));
  }
}