import 'dart:developer';
import 'dart:io';

import 'package:aretmis_cupps_runner/global/Classes.dart';
import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import '../../global/MainModel.dart';
import '../../global/MyController.dart';

class CuppsController extends MyController {
  MainModel model;

  CuppsController(this.model) : super(model: model);

  void initializeCupps({String ip = "192.168.1.248", String port="35000"}) async {


        print("Initializing Cupps $ip:$port");
        model.setPlatformStatus("Connecting to platform...");
        CUPPS
            .connectToPlatform(
                showErrorToast: false,
                applicationVersion: "1.0.0.1",
                onPlatformDisconnect: () {
                  model.setPlatformStatus("CUPPS platform is not connected!");
                },
                ip: ip,
                port: int.tryParse(port) ?? 0,
                airlineCode: "TK",
                notifier: () {
                  if (!model.bpInitialized &&
                      CUPPS().platformDevices.hasBP &&
                      CUPPS().platformDevices.bpDevices.first.icon.contains("active") &&
                      CUPPS().platformDevices.bpDevices.first.modded) {
                    model.setBpInitialized(true);
                    print("BP Initialized******${CUPPS().platformDevices.bpDevices.first.icon}");
                    setBpPrintInitializers(model.printInitializers);
                  }
                  if (CUPPS().platformDevices.hasBP &&
                      (CUPPS().platformDevices.bpDevices.first.icon.contains("bp_natural") ||
                          CUPPS().platformDevices.bpDevices.first.icon.contains("bp_disconnect") ||
                          CUPPS().platformDevices.bpDevices.first.icon.contains("bp_in"))) {
                    model.setBpInitialized(false);
                    print("BP Initialized removed______${CUPPS().platformDevices.bpDevices.first.icon}");
                  }

                  if (!model.btInitialized &&
                      CUPPS().platformDevices.hasBT &&
                      CUPPS().platformDevices.btDevices.first.icon.contains("active")) {
                    model.setBtInitialized(true);
                    setBtPrintInitializers(model.printInitializers);
                  }
                  if (CUPPS().platformDevices.hasBT &&
                      (CUPPS().platformDevices.btDevices.first.icon.contains("bt_natural") ||
                          CUPPS().platformDevices.btDevices.first.icon.contains("bt_disconnect") ||
                          CUPPS().platformDevices.btDevices.first.icon.contains("bt_in"))) {
                    model.setBtInitialized(false);
                  }

                  model.setPlatformDevices(CUPPS().platformDevices);
                },
                onPlatformConnectionChange: (bool v) {
                  model.setPlatformConnection(v);
                  if (!v) {
                    model.setPlatformStatus("CUPPS platform is not connected!");
                  }
                },
                applicationName: '${Platform.isAndroid ? "Android" : "IOS"} Artemis')
            .then((value) async {
          if (value) {
            model.setPlatformStatus("Authenticating to Cupps...");
            CUPPS.authenticate().then((value) {
              if (value) {
                model.setPlatformStatus("Connected to CUPPS");
                CUPPS.setConnectionChecker(
                    interval: Duration(seconds: 45),
                    timeout: Duration(seconds: 12),
                    retryFunction: () {
                      model.setPlatformDevices(PlatformDevices.empty());
                      model.setPlatformStatus("CUPPS platform connection lost!");
                      initializeCupps();
                    });
                CUPPS.connectToAvailableNeededDevices().then((value) {
                  if (value) {
                    CuppsController(model).lockAllAvailable();
                  }
                });
              } else {
                model.setPlatformStatus("Authentication Failed");
              }
            }).timeout(Duration(seconds: 15), onTimeout: () {
              model.setPlatformStatus("Authentication Timeout!");
              model.setPlatformDevices(PlatformDevices.empty());
              initializeCupps();
            });
            // var req = InterfaceLevelsAvailableRequest(
            //     hsXsdVersion: '01.03', messageID: CUPPS().currentMessageID);
            // CUPPS.send(xml: req.toXML());
          } else {
            model.setPlatformStatus("CUPPS platform connection failed");
            Future.delayed(const Duration(seconds: 2), () {
              model.setPlatformStatus("Retry connection in 3s ...");
            });
            Future.delayed(const Duration(seconds: 3), () {
              model.setPlatformStatus("Retry connection in 2s ...");
            });
            Future.delayed(const Duration(seconds: 4), () {
              model.setPlatformStatus("Retry connection in 1s ...");
            });
            Future.delayed(const Duration(seconds: 5), () {
              initializeCupps();
            });
          }
        }).timeout(const Duration(seconds: 45), onTimeout: () {
          model.setPlatformStatus("CUPPS platform connection timeout!");
        });

  }

  void handleAeaCommand(dynamic data, {bool print = true}) {
    if (data == null) return;
    try {
      // AeaCommandData aeaCommands = AeaCommandData.fromJson(data);
      // if (aeaCommands != null) {
      //   List<String> bps = aeaCommands.prints.firstWhere((element) => element.type == 0, orElse: () => AeaPrint(data: [])).data;
      //   List<String> bts = aeaCommands.prints.firstWhere((element) => element.type == 1, orElse: () => AeaPrint(data: [])).data;
      //   log("*** total BP Print commands: ${bps.length}");
      //   log("*** total BT Print commands: ${bts.length}");
      //   printBps(bps);
      //   printBts(bts);
      // }
    } catch (e) {
      log("$e");
      return;
    }
  }

  Future<void> setBpPrintInitializers(AeaCommandData printInitializers) async {
    log("setBpPrintInitializers Called");
    if ( model.platformDevices.bpDevices.where((element) => element.modded).isNotEmpty) {
      List<String> bpCommands = printInitializers.prints.firstWhere((element) => element.type == 0).data.map((e) => e
          .replaceAll("\r", "")).toList();

      await model.platformDevices.bpDevices
          .where((element) => element.modded)
          .first
          .completeConfigure(bpCommands);
    }
  }

  Future<void> setBtPrintInitializers(AeaCommandData printInitializers) async {
    log("setBtPrintInitializers Called");
    if (model.platformDevices.btDevices.where((element) => element.modded).isNotEmpty) {
      List<String> btCommands = printInitializers.prints.firstWhere((element) => element.type == 1).data;
      await model.platformDevices.btDevices
          .where((element) => element.modded)
          .first
          .completeConfigure(btCommands);
    }
  }

  Future<void> printBps(List<String> bps) async {
    List<String> tmpBps = bps.map((e) => e).toList();
    bool dialogShow = true;
    if (model.platformDevices.bpDevices.where((element) => element.isConnected && element.initialized).isNotEmpty) {

      for (var bp in bps) {
        CuppsCommandResponse bpPrintRes = await printBP(bp);
        if (bpPrintRes.status) {
          tmpBps.remove(bp);
        }
        if (!model.platformDevices.bpDevices.first.isPrinting && tmpBps.isNotEmpty && dialogShow) {
          dialogShow = false;
          showFailedPrintsDialog(tmpBps);
        }
      }
    }
  }

  Future<CuppsCommandResponse> printBP(String bp) async {
    if (model.platformDevices.bpDevices.where((element) => element.isConnected && element.initialized).isNotEmpty) {
      CuppsCommandResponse bpPrintRes =
          await model.platformDevices.bpDevices.firstWhere((element) => element.isConnected && element.initialized).print(bp);
      if (!bpPrintRes.status && !bpPrintRes.msg.contains("Timeout")) {
        await setBpPrintInitializers(model.printInitializers);
        CuppsCommandResponse secondTryRes =
            await model.platformDevices.bpDevices.firstWhere((element) => element.isConnected && element.initialized).print(bp);
        if (!secondTryRes.status) {
          showGetToast(secondTryRes.status ? "Done" : "Print Error", "Boarding pass print Failed:" + secondTryRes.msg,
              color: bpPrintRes.status ? Colors.green : Colors.red);
        }
        return secondTryRes;
      } else if (!bpPrintRes.status) {
        showGetToast(bpPrintRes.status ? "Done" : "Print Error", "Boarding pass print Failed:" + bpPrintRes.msg,
            color: bpPrintRes.status ? Colors.green : Colors.red);
        return bpPrintRes;
      } else {
        return bpPrintRes;
      }
    } else {
      return CuppsCommandResponse(status: false, msg: "Boarding pass print is not Ready");
    }
  }

  Future<void> printBts(List<String> bts) async {
    List<String> tmpBts = bts.map((e) => e).toList();
    bool dialogShow = true;
    if (model.platformDevices.btDevices.where((element) => element.isConnected && element.initialized).isNotEmpty) {
      for (var bt in bts) {
        CuppsCommandResponse btPrintRes = await printBT(bt);
        if (btPrintRes.status) {
          tmpBts.remove(bt);
        }
        if (!model.platformDevices.btDevices.first.isPrinting && tmpBts.isNotEmpty && dialogShow) {
          dialogShow = false;
          showFailedPrintsDialog(tmpBts,isBT:true);
        }
      }




      // bts.where((element) => element.isNotEmpty).forEach((bt) async {
      //   // final res= await model.platformDevices.btDevices.first.print(bt);
      //   CuppsCommandResponse btPrintRes = await printBT(bt);
      //   if (btPrintRes.status) {
      //     tmpBts.remove(bt);
      //   }
      //   if (!model.platformDevices.btDevices.first.isPrinting && tmpBts.isNotEmpty && dialogShow) {
      //     dialogShow = false;
      //     showFailedPrintsDialog(tmpBts, isBT: true);
      //   }
      // }

      // );
    }
  }

  Future<CuppsCommandResponse> printBT(String bt) async {
    if (model.platformDevices.btDevices.where((element) => element.isConnected && element.initialized).isNotEmpty) {
      CuppsCommandResponse btPrintRes =
          await model.platformDevices.btDevices.firstWhere((element) => element.isConnected && element.initialized).print(bt);
      if (!btPrintRes.status && !btPrintRes.msg.contains("Timeout")) {
        await setBtPrintInitializers(model.printInitializers);
        CuppsCommandResponse secondTryRes =
            await model.platformDevices.btDevices.firstWhere((element) => element.isConnected && element.initialized).print(bt);
        if (!secondTryRes.status) {
          showGetToast(secondTryRes.status ? "Done" : "Print Error", "BagTag print Failed:" + secondTryRes.msg,
              color: btPrintRes.status ? Colors.green : Colors.red);
        }
        return secondTryRes;
      } else if (!btPrintRes.status) {
        showGetToast(btPrintRes.status ? "Done" : "Print Error", "BagTag print Failed:" + btPrintRes.msg,
            color: btPrintRes.status ? Colors.green : Colors.red);
        return btPrintRes;
      } else {
        return btPrintRes;
      }
    } else {
      return CuppsCommandResponse(status: false, msg: "BagTag print is not Ready");
    }
  }

  Future<void> printPr(String text) async {
    if (model.platformDevices.prDevices.isNotEmpty) {
      final res = await model.platformDevices.prDevices.first.printPr(text);
      if (!res.status) {
        showGetToast(res.status ? "Done" : "Error", res.msg, color: res.status ? Colors.green : Colors.red);
      }
    }
  }

  showFailedPrintsDialog(List<String> failedPrints, {bool isBT = false}) {
    // showDialog(context: Get.context, builder: (c)=>FailedPrintsCuppsDialog(isBT: isBT,failedPrints: failedPrints));
    if (isBT) {
      Get.snackbar("Print Error", "",
          backgroundColor: Colors.red.withOpacity(0.65),
          duration: Duration(seconds: 20),
          titleText: Row(
            children: [
              Expanded(
                child: Text(
                  "${failedPrints.length} of BagTags Prints Failed!",
                ),
              ),
              TextButton(
                  style: TextButton.styleFrom(backgroundColor: Colors.grey),
                  onPressed: () => Get.back(), child: Text("Cancel",style: TextStyle(color: Colors.black87))),
              TextButton(
                  style: TextButton.styleFrom(backgroundColor: Colors.blue),
                  onPressed: () {
                    printBts(failedPrints);
                    Get.back();
                  },
                  child: Text("Reprint",style: TextStyle(color: Colors.white),))
            ],
          ) );
    } else {
      Get.snackbar("Print Error", "",
          backgroundColor: Colors.red.withOpacity(0.65),
          duration: Duration(seconds: 20),
          titleText: Row(
            children: [
              Expanded(
                child: Text(
                  "${failedPrints.length} of Boarding Pass Prints Failed!",
                ),
              ),
              TextButton(
                  style: TextButton.styleFrom(backgroundColor: Colors.grey),
                  onPressed: () => Get.back(), child: Text("Cancel",style: TextStyle(color: Colors.black87))),
              TextButton(
                  style: TextButton.styleFrom(backgroundColor: Colors.blue),
                  onPressed: () {
                    printBps(failedPrints);
                    Get.back();
                  },
                  child: Text("Reprint",style: TextStyle(color: Colors.white),))
            ],
          ) );
    }
  }

  readLastConnectedIpFromSp(void Function(String, String) onDone) async {

  }

  saveAddressToSP(String ip, String port) async {

  }

  Future<void> unlockAllAvailable() async {
    log("unlockAllAvailable");
    model.platformDevices.bcDevices
        .where((element) => element.isConnected && element.initialized && element.locked)
        .forEach((dev) {
      dev.unlock().then((value) {
        log("$value");
      });
    });
    model.platformDevices.ocDevices
        .where((element) => element.isConnected && element.initialized && element.locked)
        .forEach((dev) {
      dev.unlock().then((value) {
        log("$value");
      });
    });
    model.platformDevices.msDevices
        .where((element) => element.isConnected && element.initialized && element.locked)
        .forEach((dev) {
      dev.unlock().then((value) {
        log("$value");
      });
    });
    model.platformDevices.prDevices
        .where((element) => element.isConnected && element.initialized && element.locked)
        .forEach((dev) {
      dev.unlock().then((value) {
        log("$value");
      });
    });
  }

  Future<void> lockAllAvailable() async {
    log("lockAllAvailable");
    model.platformDevices.bcDevices.where((element) => element.isConnected && element.initialized).forEach((dev) {
      dev.lock().then((value) {
        log("$value");
      });
    });
    model.platformDevices.ocDevices.where((element) => element.isConnected && element.initialized).forEach((dev) {
      dev.lock().then((value) {
        log("$value");
      });
    });
    model.platformDevices.msDevices.where((element) => element.isConnected && element.initialized).forEach((dev) {
      dev.lock().then((value) {
        log("$value");
      });
    });
    model.platformDevices.prDevices.where((element) => element.isConnected && element.initialized).forEach((dev) {
      dev.lock().then((value) {
        log("$value");
      });
    });
  }

  checkDeviceAvailability({required String action,required Function onConfirm}) {
    switch (action) {
      case "Boarding":
        {
          List<BcDevice> bcs = model.platformDevices.bcDevices
              .where((element) => element.isConnected && element.initialized && element.locked)
              .toList();
          List<BgDevice> bgs =
              model.platformDevices.bgDevices.where((element) => element.isConnected && element.initialized).toList();
          if (bcs.isNotEmpty || bgs.isNotEmpty) {
            onConfirm();
          } else {
            String body = bcs.isEmpty && bgs.isEmpty
                ? "No BarcodeReader or BoardingGate is Available \n Do you want to continue?"
                : bcs.isEmpty
                    ? "BarcodeReader Device is not Available\n Do you want to continue?"
                    : "BoardingGate Device is not Available\n Do you want to continue?";
          }
          break;
        }
      case "Checkin":
        {
          List<BpDevice> bps =
              model.platformDevices.bpDevices.where((element) => element.isConnected && element.initialized).toList();
          List<BtDevice> bts =
              model.platformDevices.btDevices.where((element) => element.isConnected && element.initialized).toList();
          if (bps.isNotEmpty && bts.isNotEmpty) {
            onConfirm();
          } else {
            String body = bps.isEmpty && bts.isEmpty
                ? "BoardingPassPrinter and BagTagPrinter are not Available \n Do you want to continue?"
                : bps.isEmpty
                    ? "BoardingPassPrinter printer is not Available\n Do you want to continue?"
                    : "BagTagPrinter printer is not Available\n Do you want to continue?";

          }
          break;
        }

      case "BP":
        {
          List<BpDevice> bps =
              model.platformDevices.bpDevices.where((element) => element.isConnected && element.initialized).toList();
          if (bps.isNotEmpty) {
            onConfirm();
          } else {
            String body = "BoardingPassPrinter printer is not Available\n Do you want to continue?";
          }
          break;
        }

      case "BT":
        {
          List<BtDevice> bts =
              model.platformDevices.btDevices.where((element) => element.isConnected && element.initialized).toList();
          if (bts.isNotEmpty) {
            onConfirm();
          } else {
            String body = "BagTagPrinter printer is not Available\n Do you want to continue?";

          }
          break;
        }
      default:
        return;
    }
  }

  disconnectCupps() {
    model.setPlatformStatus("Disconnecting...");
    CUPPS.disconnectFromPlatform().then((value) {
      model.setPlatformStatus("Disconnected From Cupps");
    });
  }
}
