import 'package:aretmis_cupps_runner/global/MainModel.dart';
import 'package:flutter/material.dart';
import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:get/get.dart';
import 'package:scoped_model/scoped_model.dart';

import '../main.dart';

class DdDeviceScreen extends StatefulWidget {
  final DdDevice device;

  const DdDeviceScreen({required this.device});

  @override
  _DdDeviceScreenState createState() => _DdDeviceScreenState();
}

class _DdDeviceScreenState extends State<DdDeviceScreen> {
  @override
  Widget build(BuildContext context) {
    return ScopedModelDescendant<MainModel>(builder: (BuildContext context, Widget? child, MainModel model) {

      double width = MediaQuery.of(context).size.width;
    DdDevice device = widget.device;
    return Scaffold(
      appBar: AppBar(
        title: Text("${device.deviceIndex} ${device.deviceName}"),
        actions: [
          IconButton(
              onPressed: () async {
                List<LogEntity> data = await Logger.getLogList();
                String logDate = DateTime.now().toString().substring(0, 10);
                List<LogEntity> logs = data
                    .where(
                        (d) => d.type.toLowerCase() == device.getType.name.toLowerCase().toLowerCase() && d.dateStr == logDate)
                    .toList();
                if(logs.isNotEmpty){
                  logs.first.download();
                }else{
                  showGetToast("No Log Available","No Log for today",color: Colors.orange);
                }

              },
              icon: const Icon(Icons.bug_report_outlined))
        ],
      ),
      body: Column(
        children: [
          DeviceStatusMessageWidget(device:device),
          Expanded(
            child: Container(
              padding: const EdgeInsets.symmetric(horizontal: 12, vertical: 12),
              child: Column(
                children: [
                  Row(
                    children: [
                      Expanded(flex: 1, child: Image.asset(device.icon, package: 'artemis_cupps')),
                      const SizedBox(width: 12),
                      Expanded(
                          flex: 2,
                          child: Column(
                            children: [
                              Row(
                                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                children: [
                                  const Text("Status"),
                                  Text(
                                    "${device.deviceParameter.status}",
                                    style: const TextStyle(fontWeight: FontWeight.bold),
                                  ),
                                ],
                              ),
                              const Divider(),
                              Row(
                                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                children: [
                                  const Text("Address"),
                                  Text(
                                    "${device.deviceParameter.ipAndPort}",
                                    style: const TextStyle(fontWeight: FontWeight.bold),
                                  ),
                                ],
                              ),
                              const Divider(),
                              Row(
                                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                                children: [const Text("TCP Connection"), Icon(Icons.circle, color: device.isConnected ? Colors.green : Colors.red, size: 14)],
                              ),
                            ],
                          ))
                    ],
                  ),
                  // Row(
                  //   children: [
                  //     const Text("Available Modes"),
                  //     const SizedBox(width: 12),
                  //     Expanded(
                  //       child: Wrap(
                  //         direction: Axis.horizontal,
                  //         children: device.supportedInterfaceModes.map((e) => OutlinedButton(onPressed: () {}, child: Text(e.mode))).toList(),
                  //       ),
                  //     ),
                  //   ],
                  // ),
                  const Divider(),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  });}
}
