import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';
import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:artemis_cupps/classes/Notifications/messsage_id_error_event.dart';
import 'package:artemis_cupps/classes/Requests/device_query_request.dart';
import 'package:dart_ipify/dart_ipify.dart';
import 'package:get/get.dart';
import 'package:network_info_plus/network_info_plus.dart';

import '../../classes/classes.dart';

import 'package:xml/xml.dart';

import 'logger.dart';

const minPlatformMsgID = 4294901760;

class SocketService {
  final String ip;
  final int port;

  SocketService({required this.ip, required this.port});

  String logIdentifier = "Unkown";
  String logSuffix = "Unkown";
  int messageID = 0;
  Socket? _socket;
  bool retry = true;
  void Function(CuppsBase, String)? requestHandler;
  void Function()? onConnectionClose;
  void Function()? onConnectionLost;
  String temp = "";
  final Map<int, Completer<String>> _requests = {};

  void enableRetry() {
    log("Enable Retry");
    retry = true;
  }

  void disableRetry() {
    log("$logSuffix Disabled Retry");
    retry = false;
  }

  bool get isConnected => (_socket != null);

  Future<bool> initialize({
    // required String ip,
    // required int port,
    required void Function(CuppsBase, String) rh,
    required void Function() onClose,
    required void Function() onLost,
    required String logID,
    required String logSuff,
  }) async {
    // if (_socket != null) {
    //   await _socket!.close();
    // }
    messageID = 1;
    requestHandler = rh;
    onConnectionClose = onClose;
    logIdentifier = logID;
    logSuffix = logSuff;
    onConnectionLost = onLost;
    try {
      // late RawSocketOption option;
      // if (Platform.isAndroid) {
      //   option = RawSocketOption.fromBool(/* SOL_SOCKET */ 0x1, /* SO_KEEPALIVE */ 0x0009, true);
      // } else {
      //   // option = RawSocketOption.fromBool(/* SOL_SOCKET */ 0xffff, /* SO_KEEPALIVE */ 0x0008, true);
      //   option = RawSocketOption.fromInt(/* SOL_SOCKET */ 0xffff, /* SO_KEEPALIVE */ 0x0008, 5);
      // }
      log("Reconnecting socket");
      Socket s = await Socket.connect(ip, port, timeout: const Duration(seconds: 10)).catchError((e) {
        throw Exception("$e");
      }).timeout(const Duration(seconds: 12), onTimeout: () {
        throw Exception("TimeOut");
      });
      s.listen(_onSocketMessage, cancelOnError: true, onError: (e) {
        log("ERROR ON SOCKET $e");
        disconnect();
      }, onDone: () {
        log("Socket DONE");
        disconnect();
      });
      log("Socket set");
      enableRetry();
      _socket = s;
      return true;
    } catch (e) {
      log("ERROR ON LISTENING TO SOCKET $e");
      return false;
    }
  }

  Future<String> sendSocketMessage(String data, {bool shouldLog = true}) async {
    try {
      if (isConnected && _socket != null) {
        final completer = Completer<String>();
        final requestId = messageID;
        messageID = (messageID % minPlatformMsgID) + 1;
        _requests[requestId] = completer;
        XmlDocument formattedDOC = XmlDocument.parse(data);

        String formattedXML = formattedDOC.toXmlString(pretty: true, preserveWhitespace: (_) => true);
        String header = generateHeaderForXML(formattedXML);
        String request = header + formattedXML;
        // print("Sending $messageID --->$data");
        if (shouldLog) {
          await Logger.writeToLog(formattedXML, from: "Application", to: logIdentifier, logSuffix: logSuffix);
        }
        _socket!.write(request);
        if (shouldLog) {
          await Logger.generalLog("Socked Sent Message with ID ${messageID-1} from $logIdentifier");
        }

        return completer.future;
      } else {
        String xmlRes = '''<?xml version="1.0"?><!--Message sending time 2021-11-18 00:19:29.698-->
<cupps xmlns="http://www.cupps.aero/cupps/01.03" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" messageID="6" messageName="deviceReleaseResponse">
   <response result="Socket Closed"></response>
</cupps>''';
        return xmlRes;
      }
    } catch (e) {
      log("Sending to Socket ERROR: $e");
      rethrow;
    }
  }

  Future<void> respond(String data) async {
    try {
      if (isConnected) {
        String header = generateHeaderForXML(data);
        String request = header + data;

        XmlDocument formattedDOC = XmlDocument.parse(data);

        String formattedXML = formattedDOC.toXmlString(pretty: true, preserveWhitespace: (_) => true);

        await Logger.writeToLog(formattedXML, from: "Application", to: logIdentifier, logSuffix: logSuffix);
        _socket!.write(request);
      } else {
        log("Socket Closed");
      }
    } catch (e) {
      log("responding to Socket ERROR: $e");
      rethrow;
    }
  }

  Future<void> _onSocketMessage(data) async {
    try {
      List<String> xmls = [];
      String wholeXML = utf8.decode(data);
      await Logger.generalLog("Socked Received Data on $logIdentifier");
      bool messageENDED = false;
      // log("whole\n"+wholeXML+"\nwhole");
      if ((temp + wholeXML).contains("<cupps") && wholeXML.contains("</cupps>") && wholeXML.endsWith("</cupps>")) {
        xmls = wholeXML.split("</cupps>").where((element) => element.isNotEmpty).map((e) => "$e</cupps>").toList();
        messageENDED = true;
      } else {
        messageENDED = false;
        log("MESSAGE IS COMING");
        temp += wholeXML;
      }

      for (var xml in xmls) {
        String headerShouldBe = (temp + xml)
            .split("<")
            .first;


        String received = (temp + xml).replaceFirst(headerShouldBe, "");
        // print("received $received from Platform");
        if (messageENDED && headerShouldBe.isNotEmpty) {
          received = received.substring(0, lengthFromHeader(headerShouldBe));
        }
        try {
          if (messageENDED) {
            // print("MESSAGE COMPLETE");
            temp = "";
            String beautifulXMl = XmlDocument.parse(received).toXmlString(pretty: true, preserveWhitespace: (_) => true);
            await Logger.writeToLog(beautifulXMl, to: "Application", from: logIdentifier, logSuffix: logSuffix);
            // await Logger.generalLog("Received Data:\n$beautifulXMl");
            CuppsBase response = CuppsBase.fromXML(received);
            int messageID = response.messageID;
            await Logger.generalLog("Received Message with ID $messageID on $logIdentifier");
            if (_requests.containsKey(messageID)) {
              _requests[messageID]!.complete(received);
              _requests.remove(messageID);
            } else if (messageID >= minPlatformMsgID) {
              requestHandler!(response, received);
            } else {
              String msgIDExeptionXML = MessageIDErrorEvent(messageID: messageID, messageName: 'messageIDErrorEvent').toXML();
              sendSocketMessage(msgIDExeptionXML);
              disconnect();
            }
          } else {
            // log("MESSAGE IS COMING");
            // temp += wholeXML;
          }
        } catch (e) {
          await Logger.generalLog("Received Data Exception :$e");
          // log("$e--->\n$received\n *************************_________________________\n$xml\n*************************_________________________");
        }
      }
    }catch(e) {
      await Logger.generalLog("Data Parse Exception :$e");
    }
  }

  static String generateHeaderForXML(String xml) {
    int length = utf8.encode(xml).length;
    String temp = length.toRadixString(16).padLeft(6, "0");
    String header = "0100" + temp;
    return header;
  }

  static int lengthFromHeader(String header) {
    // log("Length From header $header");
    String value = header.substring(4, 10);
    int l = int.tryParse(value, radix: 16) ?? 0;

    return l;
  }

  Future<void> disconnect({bool disableRetry = false}) async {
    try {
      if (_socket != null) {
        if (disableRetry) {
          log("$logSuffix Disabled Retry");
          retry = false;
        }
        await _socket?.close();
        log("Socket Service Closed");
        _socket?.destroy();
        log("Socket Service Destroyed");
        _socket = null;
        onConnectionClose!();
      }
    } catch (e) {
      onConnectionClose!();
      log("Disconnect Exception $e");
    }
  }

  static Future<String?> getLocalIp() {
    return NetworkInfo().getWifiIP();
  }

  static Future<String?> getPublicIp() {
    return Ipify.ipv4();
  }

  void setConnectionChecker(
      {Duration timeout = const Duration(seconds: 10),
      Duration interval = const Duration(seconds: 30),
      required Function retryFunction,
      required Function useRetryResponse,
      required bool Function() shouldCheck}) {
    // log("RETRY :$retry");
    try {
      Future.delayed(interval, () {
        if (retry) {
          if (logIdentifier == "Platform") {
            log("$logIdentifier Connection Checker ${DateTime.now()}");
            DeviceQueryRequest req = DeviceQueryRequest(messageID: messageID);
            if (isConnected) {
              if (shouldCheck()) {
                sendSocketMessage(req.toXML(), shouldLog: true).timeout(timeout, onTimeout: () {
                  log("$logIdentifier Connection Checker Timeout **Reconnect** ${DateTime.now()}");
                  retryFunction();
                  return "";
                }).then((value) {
                  if (value.isNotEmpty && !value.contains("Socket Closed")) {
                    log("$logIdentifier Connection Checker Connection OK ${DateTime.now()}");
                    setConnectionChecker(
                        timeout: timeout,
                        interval: interval,
                        retryFunction: retryFunction,
                        useRetryResponse: useRetryResponse,
                        shouldCheck: shouldCheck);
                  }
                });
              } else {
                log("Retry and Connection checker called but shouldCheck ${shouldCheck()}");
              }
            } else {
              retryFunction();
            }
          } else {
            log("$logIdentifier Connection Checker ${DateTime.now()}");
            DeviceStatusRequest req = DeviceStatusRequest(messageID: messageID);
            if (isConnected) {
              if (shouldCheck()) {
                sendSocketMessage(req.toXML(), shouldLog: false).timeout(timeout, onTimeout: () {
                  log("$logIdentifier Connection Checker Timeout **Reconnect** ${DateTime.now()}");
                  retryFunction();
                  return "";
                }).then((value) {
                  if (value.isNotEmpty && !value.contains("Socket Closed")) {
                    DeviceStatusResponse statusResponse = DeviceStatusResponse.fromXML(value);
                    useRetryResponse(statusResponse);
                    log("$logIdentifier Connection Checker Connection OK ${DateTime.now()}");
                    setConnectionChecker(
                        timeout: timeout,
                        interval: interval,
                        retryFunction: retryFunction,
                        useRetryResponse: useRetryResponse,
                        shouldCheck: shouldCheck);
                  }
                });
              } else {
                log("Retry and Connection checker called but shouldCheck ${shouldCheck()}");
              }
            } else {
              log("$logIdentifier Connection Checker called but socked is CLOSED && retry $retry");
              if (retry) {
                retryFunction();
              }
              // retryFunction();
            }
          }
        }
      });
    } catch (e) {
      log("$logIdentifier Connection Checker Error **Reconnect** ${DateTime.now()}");
      onConnectionLost!();
    }
  }
}
